from flask import Flask, jsonify, request, Response, render_template
from lib389._constants import *
from lib389.properties import *
import ldap


def jsonResponse(msg, status_code, link):
    """
    Return the json representation
    """
    count = 1
    if isinstance(msg, list):
        count = len(msg)

    resp_msg = {'href': link,
                'resultCode': status_code,
                'resultCount': count,
                'result': msg}

    resp = jsonify(resp_msg)
    resp.status_code = status_code
    return resp


def jsonGetPatchOp(mod):
    """
    Parse the modify type
    """
    if mod['op'] == 'remove':
        op = ldap.MOD_DELETE
    elif mod['op'] == 'add':
        op = ldap.MOD_ADD
    elif mod['op'] == 'replace':
        op = ldap.MOD_REPLACE
    else:
        op = -1

    return op


def getEntryRepByDN(dn):
    """
    Take a DN, search for it, and return the entire entry in a
    pre-json representation(dict)
    """
    ds = getConnection()
    try:
        entry = ds.search_s(dn, ldap.SCOPE_BASE, 'objectclass=top')
        return entry[0].getJSONEntry()
    except ldap.LDAPError as e:
        raise "Error: " + str(e)


def postEntryToRep(postentry):
    """
    Convert the post read control entry to "our" JSON representation
    """
    return {'dn': postentry.dn, 'attrs': dict(postentry.entry)}


def getSrchLink(url, basedn, scope, filter_str):
    """
    Return the link for a reaponse message
    """
    msg = '%sv1/ldap/%s/%s/%s' % (url, basedn, scope, filter_str)
    return msg


def suffixToBackend(ds, suffix):
    """
    Take a suffix string and get the backend name
    """
    backend = None
    try:
        #
        # Try the internal mapping tree first
        #
        props = ds.mappingtree.getProperties(suffix=suffix,
                                             properties=[MT_BACKEND])
        backend = props[MT_BACKEND][0]
    except ldap.NO_SUCH_OBJECT:
        #
        # Look up backend by suffix
        #
        filter_str = '(nsslapd-suffix=%s)' % suffix
        entry = ds.search_s("cn=ldbm database,cn=plugins,cn=config",
                            ldap.SCOPE_SUBTREE, filter_str)
        if entry:
            backend = entry[0].getValue('cn')

    return backend


def backendToSuffix(ds, backend):
    """
    Take a backend name and get the suffix
    """
    pass
